@@ -29,6 +29,23 @@
 
 /*
   $Log: orbParameters.h,v $
+  Revision 1.1.2.9  2005/11/16 17:35:25  dgrisby
+  New connectionWatchPeriod and connectionWatchImmediate parameters.
+
+  Revision 1.1.2.8  2005/11/14 10:57:18  dgrisby
+  Minor comment fix.
+
+  Revision 1.1.2.7  2005/08/23 11:45:06  dgrisby
+  New maxSocketSend and maxSocketRecv parameters.
+
+  Revision 1.1.2.6  2005/02/13 20:53:08  dgrisby
+  Change threadPoolWatchConnection parameter to be an integer rather
+  than a boolean. Value is the count of threads that can be handling a
+  connection when one decides to watch it.
+
+  Revision 1.1.2.5  2004/03/02 15:31:22  dgrisby
+  Support for persistent server identifier.
+
   Revision 1.1.2.4  2002/10/14 20:06:41  dgrisby
   Per objref / per thread timeouts.
 
@@ -101,6 +118,13 @@ _CORBA_MODULE_VAR _core_attr CORBA::ULong giopMaxMsgSize;
 //   Valid values = (n >= 8192)
 //
 
+_CORBA_MODULE_VAR _core_attr size_t maxSocketSend;
+_CORBA_MODULE_VAR _core_attr size_t maxSocketRecv;
+//   These values set the maximum size that is used in an individual
+//   socket send() or recv() call.
+//
+//   Valid values = (n >= 8192)
+
 
 _CORBA_MODULE_VAR _core_attr omniCodeSet::NCS_C* nativeCharCodeSet;
 //  set the native code set for char and string
@@ -304,7 +328,7 @@ _CORBA_MODULE_VAR _core_attr timeValue clientCallTimeOutPeriod;
 //   than the timeout value, the ORB will shutdown the connection and
 //   raise a COMM_FAILURE.
 //
-//   Valid values = (n >= 0 in seconds) 
+//   Valid values = (n >= 0 in milliseconds) 
 //                   0 --> no timeout. Block till a reply comes back
 
 _CORBA_MODULE_VAR _core_attr CORBA::Boolean supportPerThreadTimeOut;
@@ -369,14 +393,20 @@ _CORBA_MODULE_VAR _core_attr CORBA::ULong   maxServerThreadPoolSize;
 //
 //   Valid values = (n >= 1) 
 
-_CORBA_MODULE_VAR _core_attr CORBA::Boolean threadPoolWatchConnection;
-//   1 means that after dispatching an upcall in thread pool mode, the
-//   thread should watch the connection for a short time before
-//   returning to the pool. This leads to less thread switching for
-//   series of calls from a single client, but is less fair if there
-//   are concurrent clients.
+_CORBA_MODULE_VAR _core_attr CORBA::ULong   threadPoolWatchConnection;
+//   After dispatching an upcall in thread pool mode, the thread that
+//   has just performed the call can watch the connection for a short
+//   time before returning to the pool. This leads to less thread
+//   switching for a series of calls from a single client, but is less
+//   fair if there are concurrent clients. The connection is watched
+//   if the number of threads concurrently handling the connection is
+//   <= the value of this parameter. i.e. if the parameter is zero,
+//   the connection is never watched; if it is 1, the last thread
+//   managing a connection watches it; if 2, the connection is still
+//   watched if there is one other thread still in an upcall for the
+//   connection, and so on.
 //
-//  Valid values = 0 or 1
+//  Valid values = (n >= 0)
 
 _CORBA_MODULE_VAR _core_attr CORBA::Boolean acceptBiDirectionalGIOP;
 //   Applies to the server side. Set to 1 to indicates that the
@@ -434,10 +464,19 @@ _CORBA_MODULE_VAR _core_attr CORBA::UShort unixTransportPermission;
 //  Valid values = unix permission mode bits in octal radix (e.g. 0755)
 
 _CORBA_MODULE_VAR _core_attr CORBA::Boolean supportBootstrapAgent;
-//Applies to the server side. 1 means enable the support for Sun's
-//bootstrap agent protocol.  This enables interoperability between omniORB
-//servers and Sun's javaIDL clients. When this option is enabled, an
-//omniORB server will response to a bootstrap agent request.
+//  Applies to the server side. 1 means enable the support for Sun's
+//  bootstrap agent protocol.  This enables interoperability between omniORB
+//  servers and Sun's javaIDL clients. When this option is enabled, an
+//  omniORB server will response to a bootstrap agent request.
+
+_CORBA_MODULE_VAR _core_attr _CORBA_Unbounded_Sequence_Octet persistentId;
+//  Persistent identifier used to detect object references that should
+//  be considered to be in this process.
+
+_CORBA_MODULE_VAR _core_attr CORBA::Boolean connectionWatchImmediate;
+//  If true, connections are watched immediately while an upcall is
+//  handled; otherwise, they are not watched until the
+//  SocketCollection next scans the connection list.
 
 _CORBA_MODULE_END
 
