@@ -29,6 +29,18 @@
 
 /*
   $Log: omniORB.h,v $
+  Revision 1.2.2.23  2005/09/19 14:23:56  dgrisby
+  New traceFile configuration parameter.
+
+  Revision 1.2.2.22  2005/03/14 13:38:26  dgrisby
+  Add omniORB::versionString() and versionHex() functions.
+
+  Revision 1.2.2.21  2004/07/23 13:25:44  dgrisby
+  New traceExceptions option.
+
+  Revision 1.2.2.20  2004/03/02 15:31:22  dgrisby
+  Support for persistent server identifier.
+
   Revision 1.2.2.19  2002/10/14 20:06:03  dgrisby
   Per objref / per thread timeouts.
 
@@ -198,6 +210,23 @@ _CORBA_MODULE omniORB
 
 _CORBA_MODULE_BEG
 
+  ///////////////////////////////////////////////////////////////////////
+  // omniORB version                                                   //
+  //                                                                   //
+  // versionString()                                                   //
+  //   Return the omniORB version as a string, e.g. 4.0.6. The caller  //
+  //   must not free the string.                                       //
+  _CORBA_MODULE_FN const char* versionString();                        //
+  //                                                                   //
+  // versionHex()                                                      //
+  //   Return the omniORB version as a hex integer. e.g. for omniORB   //
+  //   4.0.6 final, the hex string is 0x040006f0.                      //
+  //   The last byte represents the patch level -- a1 means alpha 1;   //
+  //   b2 means beta 2; c3 means release candidate 3; f0 means final;  //
+  //   f1 means final patch level 1, and so on.                        //
+  //                                                                   //
+  _CORBA_MODULE_FN _CORBA_ULong versionHex();                          //
+  ///////////////////////////////////////////////////////////////////////
 
   ///////////////////////////////////////////////////////////////////////
   // Tracing level                                                     //
@@ -218,6 +247,13 @@ _CORBA_MODULE_BEG
   //     option: -ORBtraceLevel <n>, or by direct assignment to this   //
   //     variable.                                                     //
   //                                                                   //
+  _CORBA_MODULE_VAR _core_attr _CORBA_Boolean traceExceptions;         //
+  //                                                                   //
+  //     This value can be changed at runtime either by command-line   //
+  //     option: -ORBtraceExceptions, or by direct assignment to this  //
+  //     variable.  If true, then system exceptions are logged when    //
+  //     they are thrown.                                              //
+  //                                                                   //
   _CORBA_MODULE_VAR _core_attr _CORBA_Boolean traceInvocations;        //
   //                                                                   //
   //     This value can be changed at runtime either by command-line   //
@@ -303,6 +339,18 @@ _CORBA_MODULE_BEG
   _CORBA_MODULE_FN _CORBA_ULong giopMaxMsgSize();                       //
   ////////////////////////////////////////////////////////////////////////
 
+  ////////////////////////////////////////////////////////////////////////
+  //                                                                    //
+  // setPersistentServerIdentifier()                                    //
+  //                                                                    //
+  // Sets an octet sequence used to persistently identify "this"        //
+  // server. Stored object references matching this identifier are      //
+  // re-written to use the current endpoint details.                    //
+  //                                                                    //
+  _CORBA_MODULE_FN void setPersistentServerIdentifier(                  //
+                            const _CORBA_Unbounded_Sequence_Octet& id); //
+  ////////////////////////////////////////////////////////////////////////
+
 
   //////////////////////////////////////////////////////////////////////
   /////////////////////////// omniORB::logger //////////////////////////
@@ -323,6 +371,19 @@ _CORBA_MODULE_BEG
   ////////////////////////////////////////////////////////////////////////
 
 
+  ////////////////////////////////////////////////////////////////////////
+  //                                                                    //
+  // setLogFilename()                                                   //
+  //                                                                    //
+  // Set a logging file to use instead of stderr. The file is opened    //
+  // for appending. If the file cannot be opened, CORBA::INITIALIZE is  //
+  // thrown.                                                            //
+  //                                                                    //
+  _CORBA_MODULE_FN void setLogFilename(const char* n);                  //
+  _CORBA_MODULE_FN const char* getLogFilename();                        //
+  ////////////////////////////////////////////////////////////////////////
+
+
   class logger {
   public:
     logger(const char* prefix = 0);  // prefix defaults to "omniORB: "
