@@ -28,6 +28,12 @@
 
 /*
  $Log: seqTemplatedecls.h,v $
+ Revision 1.1.2.10  2005/10/11 13:17:01  dgrisby
+ Win64 support, thanks Peter Klotz. sizeof(long) < sizeof(void*) !!
+
+ Revision 1.1.2.9  2004/04/30 15:58:33  dgrisby
+ Make ptr_arith_t unsigned.
+
  Revision 1.1.2.8  2003/01/16 12:47:08  dgrisby
  Const cast macro. Thanks Matej Kenda.
 
@@ -1901,9 +1907,11 @@ public:
   }
 
 #if SIZEOF_PTR == SIZEOF_LONG
-  typedef long ptr_arith_t;
+  typedef unsigned long ptr_arith_t;
 #elif SIZEOF_PTR == SIZEOF_INT
-  typedef int ptr_arith_t;
+  typedef unsigned int ptr_arith_t;
+#elif defined (_WIN64)
+  typedef size_t ptr_arith_t;
 #else
 #error "No suitable type to do pointer arithmetic"
 #endif
