@@ -29,6 +29,12 @@
 
 /*
  $Log: stringtypes.h,v $
+ Revision 1.2.2.16  2005/10/11 13:17:01  dgrisby
+ Win64 support, thanks Peter Klotz. sizeof(long) < sizeof(void*) !!
+
+ Revision 1.2.2.15  2004/04/30 15:58:33  dgrisby
+ Make ptr_arith_t unsigned.
+
  Revision 1.2.2.14  2003/11/06 10:17:35  dgrisby
  Remove calls to strlen in operator[]. Why were they ever considered a
  good idea?
@@ -648,9 +654,11 @@ public:
   }
 
 #if SIZEOF_PTR == SIZEOF_LONG
-  typedef long ptr_arith_t;
+  typedef unsigned long ptr_arith_t;
 #elif SIZEOF_PTR == SIZEOF_INT
-  typedef int ptr_arith_t;
+  typedef unsigned int ptr_arith_t;
+#elif defined (_WIN64)
+  typedef size_t ptr_arith_t;
 #else
 #error "No suitable type to do pointer arithmetic"
 #endif
