@@ -29,6 +29,15 @@
 
 /*
   $Log: wstringtypes.h,v $
+  Revision 1.1.2.14  2005/10/11 13:17:01  dgrisby
+  Win64 support, thanks Peter Klotz. sizeof(long) < sizeof(void*) !!
+
+  Revision 1.1.2.13  2005/03/14 13:39:51  dgrisby
+  Avoid compiler warnings about empty for loop.
+
+  Revision 1.1.2.12  2004/04/30 15:58:33  dgrisby
+  Make ptr_arith_t unsigned.
+
   Revision 1.1.2.11  2003/11/06 10:17:36  dgrisby
   Remove calls to strlen in operator[]. Why were they ever considered a
   good idea?
@@ -96,7 +105,7 @@ static inline void free(_CORBA_WChar* s) {
 
 static inline int len(const _CORBA_WChar* s) {
   int i;
-  for (i=0; *s; s++, i++);
+  for (i=0; *s; s++, i++) {};
   return i;
 }
 
@@ -618,9 +627,11 @@ public:
   }
 
 #if SIZEOF_PTR == SIZEOF_LONG
-  typedef long ptr_arith_t;
+  typedef unsigned long ptr_arith_t;
 #elif SIZEOF_PTR == SIZEOF_INT
-  typedef int ptr_arith_t;
+  typedef unsigned int ptr_arith_t;
+#elif defined (_WIN64)
+  typedef size_t ptr_arith_t;
 #else
 #error "No suitable type to do pointer arithmetic"
 #endif
