@@ -29,6 +29,19 @@
 
 /*
   $Log: GIOP_S.cc,v $
+  Revision 1.1.4.29  2005/11/15 11:07:57  dgrisby
+  More shutdown cleanup.
+
+  Revision 1.1.4.28  2005/04/08 00:06:15  dgrisby
+  Remove all remaining uses of logf.
+
+  Revision 1.1.4.27  2005/01/04 18:09:29  dgrisby
+  SkipRequestBody did not call notifyWkPreUpCall, which meant requests
+  would be mislaid if they came in the same buffer as a skipped request.
+
+  Revision 1.1.4.26  2004/10/17 21:48:38  dgrisby
+  Support CancelRequest better.
+
   Revision 1.1.4.25  2003/07/16 14:22:38  dgrisby
   Speed up oneway handling a little. More tracing for split messages.
 
@@ -214,7 +227,6 @@ GIOP_S::dispatcher() {
     impl()->inputMessageBegin(this,impl()->unmarshalWildCardRequestHeader);
 
     {
-      ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,0);
       omni_tracedmutex_lock sync(*omniTransportLock);
       pd_state = RequestHeaderIsBeingProcessed;
       if (!pd_strand->stopIdleCounter()) {
@@ -324,9 +336,14 @@ GIOP_S::handleRequest() {
 
     // Oh dear.
 
-    OMNIORB_THROW(OBJECT_NOT_EXIST,OBJECT_NOT_EXIST_NoMatch,
-		  CORBA::COMPLETED_NO);
-
+    if (omniObjAdapter::isDeactivating())
+      OMNIORB_THROW(OBJ_ADAPTER,
+		    OBJ_ADAPTER_POAUnknownAdapter,
+		    CORBA::COMPLETED_NO);
+    else
+      OMNIORB_THROW(OBJECT_NOT_EXIST,
+		    OBJECT_NOT_EXIST_NoMatch,
+		    CORBA::COMPLETED_NO);
   }
   catch(omniORB::LOCATION_FORWARD& ex) {
     // This is here to provide a convenient way to implement
@@ -334,9 +351,11 @@ GIOP_S::handleRequest() {
     // a location forward exception to re-direct the request
     // to another location.
 
-    if( omniORB::traceInvocations )
-      omniORB::logf("Implementation of \'%s\' generated LOCATION_FORWARD.",
-		    operation());
+    if( omniORB::traceInvocations ) {
+      omniORB::logger l;
+      l << "Implementation of '" << operation()
+	<< "' generated LOCATION_FORWARD.\n";
+    }
 
     CORBA::Object_var release_it(ex.get_obj());
     if (pd_state == RequestIsBeingProcessed) {
@@ -551,9 +570,11 @@ CORBA::Boolean
 GIOP_S::handleCancelRequest() {
   // We do not have the means to asynchronously abort the execution of
   // an upcall by another thread. Therefore it is not possible to
-  // cancel a request that has already been in progress. 
-  omniORB::logs(5, "Received and ignored a CancelRequest message.");
-  pd_state = ReplyCompleted;
+  // cancel a request that has already been in progress. The best we
+  // can do is prevent the reply from happening.
+  omniORB::logs(5, "Received a CancelRequest message.");
+  pd_state = WaitingForReply;
+  response_expected(0);
   return 1;
 }
 
@@ -617,6 +638,14 @@ GIOP_S::SkipRequestBody() {
   OMNIORB_ASSERT(pd_state == RequestIsBeingProcessed);
 
   pd_state = WaitingForReply;
+
+  CORBA::Boolean data_in_buffer = 0;
+  if (pd_rdlocked) {
+    giopStrand& s = (giopStrand&) *this;
+    data_in_buffer = ((s.head) ? 1 : 0);
+  }
+  pd_worker->server()->notifyWkPreUpCall(pd_worker,data_in_buffer);
+
   impl()->inputMessageEnd(this,1);
 }
 
