@@ -29,6 +29,15 @@
 
 /*
   $Log: corbaOrb.cc,v $
+  Revision 1.33.2.48  2005/11/28 14:57:51  dgrisby
+  New abortOnNativeException parameter for Windows.
+
+  Revision 1.33.2.47  2005/09/08 14:26:17  dgrisby
+  New -ORBconfigFile command line argument.
+
+  Revision 1.33.2.46  2005/03/14 13:36:02  dgrisby
+  Call detach() not attach() on shutdown!
+
   Revision 1.33.2.45  2003/08/21 14:57:44  dgrisby
   Really silly bug broke registry reading on Windows.
 
@@ -389,6 +398,14 @@ static CORBA::Boolean       invoker_shutting_down     = 0;
 extern "C" int sigaction(int, const struct sigaction *, struct sigaction *);
 #endif
 
+// Config file
+#if defined(NTArchitecture) && !defined(__ETS_KERNEL__)
+static const char* config_fname = 0;
+#else
+static const char* config_fname = CONFIG_DEFAULT_LOCATION;
+#endif
+
+
 
 ///////////////////////////////////////////////////////////////////////
 //          Per module initialisers.
@@ -515,7 +532,7 @@ CORBA::ORB_init(int& argc, char** argv, const char* orb_identifier,
   const char* option_src_3 = "argument";
   const char* option_src_4 = "option list";
   const char* option_src_5 = "-ORB arguments";
-  const char* option_source;
+  const char* option_source= 0;
   try {
 
     orbOptions::singleton().reset();
@@ -524,18 +541,17 @@ CORBA::ORB_init(int& argc, char** argv, const char* orb_identifier,
     option_source = option_src_5;
     orbOptions::singleton().getTraceLevel(argc,argv);
 
-    // Parse configuration file
-    option_source = option_src_1;
-
-#if defined(NTArchitecture) && !defined(__ETS_KERNEL__)
-    const char* config_fname = 0;
-#else
-    const char* config_fname = CONFIG_DEFAULT_LOCATION;
-#endif
     {
       const char* f = getenv(CONFIG_ENV);
       if (f) config_fname = f;
     }
+    // Configuration file name can be overriden by command line.
+    config_fname = orbOptions::singleton().getConfigFileName(argc, argv,
+							     config_fname);
+
+    // Parse configuration file
+    option_source = option_src_1;
+
     if (config_fname) {
       orbOptions::singleton().importFromFile(config_fname);
     }
@@ -620,10 +636,10 @@ CORBA::ORB_init(int& argc, char** argv, const char* orb_identifier,
     // among the modules.
     omni_giopEndpoint_initialiser_.attach();
     omni_transportRules_initialiser_.attach();
+    omni_interceptor_initialiser_.attach();
     omni_omniInternal_initialiser_.attach();
     omni_corbaOrb_initialiser_.attach();
     omni_objadpt_initialiser_.attach();
-    omni_interceptor_initialiser_.attach();
     omni_giopStreamImpl_initialiser_.attach();
     omni_omniIOR_initialiser_.attach();
     omni_ior_initialiser_.attach();
@@ -881,9 +897,9 @@ omniOrbORB::destroy()
 
     // Call detach method of the initialisers in reverse order.
     omni_hooked_initialiser_.detach();
-    omni_uri_initialiser_.attach();
-    omni_poa_initialiser_.attach();
-    omni_orbOptions_initialiser_.attach();
+    omni_uri_initialiser_.detach();
+    omni_poa_initialiser_.detach();
+    omni_orbOptions_initialiser_.detach();
     omni_initRefs_initialiser_.detach();
     omni_ObjRef_initialiser_.detach();
     omni_dynamiclib_initialiser_.detach();
@@ -898,10 +914,10 @@ omniOrbORB::destroy()
     omni_ior_initialiser_.detach();
     omni_omniIOR_initialiser_.detach();
     omni_giopStreamImpl_initialiser_.detach();
-    omni_interceptor_initialiser_.detach();
     omni_objadpt_initialiser_.detach();
     omni_corbaOrb_initialiser_.detach();
     omni_omniInternal_initialiser_.detach();
+    omni_interceptor_initialiser_.detach();
     omni_transportRules_initialiser_.detach();
     omni_giopEndpoint_initialiser_.detach();
 
@@ -1468,6 +1484,30 @@ public:
 
 static poa_iiop_name_portHandler poa_iiop_name_portHandler_;
 
+/////////////////////////////////////////////////////////////////////////////
+class configFileHandler : public orbOptions::Handler {
+public:
+
+  configFileHandler() :
+    orbOptions::Handler("configFile",
+			"configFile = <filename>",
+			1,
+			"-ORBconfigFile <filename>") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+    // Do nothing -- already handled before normal arguments are processed
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    orbOptions::addKVString(key(), config_fname ? config_fname : "[none]",
+			    result);
+  }
+};
+
+static configFileHandler configFileHandler_;
+
+
 /////////////////////////////////////////////////////////////////////////////
 //            Module initialiser                                           //
 /////////////////////////////////////////////////////////////////////////////
@@ -1483,6 +1523,7 @@ public:
     orbOptions::singleton().registerHandler(principalHandler_);
     orbOptions::singleton().registerHandler(poa_iiop_portHandler_);
     orbOptions::singleton().registerHandler(poa_iiop_name_portHandler_);
+    orbOptions::singleton().registerHandler(configFileHandler_);
   }
 
 
