@@ -29,6 +29,31 @@
 
 /*
   $Log: giopImpl12.cc,v $
+  Revision 1.1.4.30  2005/10/17 15:49:10  dgrisby
+  Missing newline at end of log message.
+
+  Revision 1.1.4.29  2005/04/10 22:17:19  dgrisby
+  Fixes to connection management. Thanks Jon Biggar.
+
+  Revision 1.1.4.28  2005/02/02 00:21:07  dgrisby
+  Memory leak with CloseConnection or error received in a queued message.
+
+  Revision 1.1.4.27  2004/10/18 00:23:54  dgrisby
+  Bug in trying to set up bidirectional GIOP with GIOP 1.0 and 1.1.
+
+  Revision 1.1.4.26  2004/07/01 19:16:23  dgrisby
+  Client call interceptor oneway and response expected flipped. Thanks
+  John Fardo.
+
+  Revision 1.1.4.25  2004/06/18 14:45:24  dgrisby
+  Obscure and rare bug in GIOP 1.2 system exception sending.
+
+  Revision 1.1.4.24  2004/05/25 14:02:22  dgrisby
+  Properly close bidirectional connections.
+
+  Revision 1.1.4.23  2004/02/06 16:17:45  dgrisby
+  Properly handle large giopMaxMsgSize settings.
+
   Revision 1.1.4.22  2003/11/19 10:42:09  dgrisby
   Locking bug with comm failure in bidirectional GIOP.
 
@@ -173,7 +198,8 @@ public:
 
   static void inputSkipWholeMessage(giopStream* g);
 
-  static void inputTerminalProtocolError(giopStream* g);
+  static void inputTerminalProtocolError(giopStream* g,
+					 const char* file, int line);
   // Helper function.  Call this function to indicate that a protocol
   // voilation was detected.  This function *always* raise a
   // giopStream::CommFailure exception.  Therefore the caller should not
@@ -195,11 +221,13 @@ private:
   giopImpl12& operator=(const giopImpl12&);
 };
 
-
 ////////////////////////////////////////////////////////////////////////
 void
 giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 
+  // On entry, this function owns the giopStream_Buffer. On exit, the
+  // buffer has either been assigned to another owner or deleted.
+
   unsigned char* hdr = (unsigned char*)b + b->start;
   GIOP::MsgType mtype = (GIOP::MsgType)hdr[7];
 
@@ -211,10 +239,19 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     // another 1.2 message. It does not say if a GIOP 1.0 or 1.1 message
     // can interleave with a GIOP 1.2 message. Our interpretation is to
     // disallow this.
-    inputTerminalProtocolError(g);
-    // never reach here
-  }
+    //
+    // *** HERE: when we support GIOP > 1.2, we can get here (or
+    // rather the equivalent place in GIOP 1.3+) due to receiving a
+    // reply message on the receiver thread of a bidirectional
+    // connection.
 
+    // We accept a CloseConnection message with any GIOP version.
+    if ((GIOP::MsgType)hdr[7] != GIOP::CloseConnection) {
+      giopStream_Buffer::deleteBuffer(b);
+      inputTerminalProtocolError(g, __FILE__, __LINE__);
+      // never reach here
+    }
+  }
 
   CORBA::ULong reqid;
 
@@ -231,23 +268,26 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     }
   }
   else if ( mtype == GIOP::MessageError) {
-    inputTerminalProtocolError(g);
+    giopStream_Buffer::deleteBuffer(b);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here
   }
   else if ( g->pd_strand->isClient() || g->pd_strand->biDir) {
     // orderly shutdown.
     CORBA::ULong minor;
     CORBA::Boolean retry;
+    giopStream_Buffer::deleteBuffer(b);
+    g->pd_strand->orderly_closed = 1;
     g->notifyCommFailure(0,minor,retry);
     g->pd_strand->state(giopStrand::DYING);
-    g->pd_strand->orderly_closed = 1;
     giopStream::CommFailure::_raise(minor,
 				    CORBA::COMPLETED_NO,
 				    retry,__FILE__,__LINE__);
     // never reach here
   }
   else {
-    inputTerminalProtocolError(g);
+    giopStream_Buffer::deleteBuffer(b);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here
   }
 
@@ -262,7 +302,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
   case GIOP::LocateReply:
     if (!(g->pd_strand->isClient() || g->pd_strand->biDir)) {
       omniTransportLock->unlock();
-      inputTerminalProtocolError(g);
+      giopStream_Buffer::deleteBuffer(b);
+      inputTerminalProtocolError(g, __FILE__, __LINE__);
       // never reach here
     }
     // falls through
@@ -282,7 +323,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	  if (target->inputFullyBuffered()) {
 	    // a reply has already been received!
 	    omniTransportLock->unlock();
-	    inputTerminalProtocolError(g);
+	    giopStream_Buffer::deleteBuffer(b);
+	    inputTerminalProtocolError(g, __FILE__, __LINE__);
 	    // never reach here
 	  }
 
@@ -290,14 +332,16 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    if (mtype != GIOP::Fragment) {
 	      // already got the header
 	      omniTransportLock->unlock();
-	      inputTerminalProtocolError(g);
+	      giopStream_Buffer::deleteBuffer(b);
+	      inputTerminalProtocolError(g, __FILE__, __LINE__);
 	      // never reach here
 	    }
 	  }
 	  else if (mtype == GIOP::Fragment) {
 	    // receive body before the header
 	    omniTransportLock->unlock();
-	    inputTerminalProtocolError(g);
+	    giopStream_Buffer::deleteBuffer(b);
+	    inputTerminalProtocolError(g, __FILE__, __LINE__);
 	    // never reach here
 	  }
 	  else {
@@ -322,7 +366,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
       {
 	if (g->pd_strand->isClient() && !g->pd_strand->biDir) {
 	  omniTransportLock->unlock();
-	  inputTerminalProtocolError(g);
+	  giopStream_Buffer::deleteBuffer(b);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__);
 	  // never reach here
 	}
 	// Make sure this request id has not been seen before
@@ -337,7 +382,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	  else if (target->requestId() == reqid) {
 	    // already have a request with the same id.
 	    omniTransportLock->unlock();
-	    inputTerminalProtocolError(g);
+	    giopStream_Buffer::deleteBuffer(b);
+	    inputTerminalProtocolError(g, __FILE__, __LINE__);
 	    // never reach here
 	  }
 	}
@@ -350,10 +396,12 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	unused->state(IOP_S::InputPartiallyBuffered);
 	unused->requestId(reqid);
       }
+      // falls through
     case GIOP::CancelRequest:
       if (g->pd_strand->isClient() && !g->pd_strand->biDir) {
 	omniTransportLock->unlock();
-	inputTerminalProtocolError(g);
+	giopStream_Buffer::deleteBuffer(b);
+	inputTerminalProtocolError(g, __FILE__, __LINE__);
 	// never reach here
       }
       // falls through
@@ -373,7 +421,8 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    if (target->inputFullyBuffered()) {
 	      // a reply has already been received!
 	      omniTransportLock->unlock();
-	      inputTerminalProtocolError(g);
+	      giopStream_Buffer::deleteBuffer(b);
+	      inputTerminalProtocolError(g, __FILE__, __LINE__);
 	      // never reach here
 	    }
 
@@ -426,10 +475,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
       if (!matched_target_is_client) {
 
 	((GIOP_S*)matched_target)->state(IOP_S::InputFullyBuffered);
-	if (omniORB::trace(40)) {
-	  omniORB::logger log;
-	  log << "Changed GIOP_S to InputFullyBuffered\n";
-	}
+	omniORB::logs(40, "Changed GIOP_S to InputFullyBuffered");
 	if (!g->pd_strand->isClient()) {
 	  g->pd_strand->server->notifyCallFullyBuffered(g->pd_strand->connection);
 	}
@@ -465,7 +511,7 @@ giopImpl12::inputNewServerMessage(giopStream* g) {
                                        g->pd_currentInputBuffer->start;
 
   if (hdr[4] != 1 || hdr[5] > 2 || hdr[7] > (unsigned char) GIOP::Fragment) {
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here
   }
 
@@ -481,7 +527,7 @@ giopImpl12::inputNewServerMessage(giopStream* g) {
       break;
     }
     else {
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__);
       // Never reach here.
     }
   case GIOP::Fragment:
@@ -526,7 +572,7 @@ giopImpl12::inputNewFragment(giopStream* g) {
       throw GIOP_S::terminateProcessing();
     }
     else {
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__);
       // never reach here.
     }
   }
@@ -535,7 +581,7 @@ giopImpl12::inputNewFragment(giopStream* g) {
 			  ? 0 : 1 );
 
   if (bswap != g->pd_unmarshal_byte_swap) {
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here
   }
 
@@ -678,6 +724,7 @@ giopImpl12::inputMessageBegin(giopStream* g,
     GIOP::Version v;
     v.major = 1;
     v.minor = hdr[5];
+    ((giopStrand &)*g).version = v;
     g->impl(giopStreamImpl::matchVersion(v));
     OMNIORB_ASSERT(g->impl());
     g->impl()->inputMessageBegin(g,g->impl()->unmarshalWildCardRequestHeader);
@@ -772,7 +819,7 @@ giopImpl12::inputMessageEnd(giopStream* g,CORBA::Boolean disgard) {
 	disgard = 1;
       }
       else {
-	inputTerminalProtocolError(g);
+	inputTerminalProtocolError(g, __FILE__, __LINE__);
 	// never reach here.
       }
     }
@@ -802,7 +849,7 @@ giopImpl12::unmarshalReplyHeader(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::Reply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // Never reach here.
   }
 
@@ -826,7 +873,7 @@ giopImpl12::unmarshalReplyHeader(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // Never reach here.
   }
 
@@ -864,7 +911,7 @@ giopImpl12::unmarshalLocateReply(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::LocateReply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // Never reach here.
   }
 
@@ -888,7 +935,7 @@ giopImpl12::unmarshalLocateReply(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here.
   }
   giop_c.locateStatus((GIOP::LocateStatusType)v);
@@ -913,14 +960,19 @@ giopImpl12::unmarshalWildCardRequestHeader(giopStream* g) {
     ((GIOP_S*)g)->requestId(reqid);
     break;
   case GIOP::CloseConnection:
-    if (g->pd_strand->biDir) {
+    if (g->pd_strand->biDir && g->pd_strand->isClient()) {
       // proper shutdown of a connection.
-      // XXX what to do?
+      if (omniORB::trace(30)) {
+	omniORB::logger l;
+	l << "Server has closed a bi-directional connection on strand "
+	  << (void*)g->pd_strand << ". Will scavenge it.\n";
+      }
+      g->pd_strand->startIdleCounter();
     }
     inputRaiseCommFailure(g);
     break;
   default:
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__);
     // Never reach here.
     break;
   }
@@ -1027,7 +1079,7 @@ giopImpl12::getInputData(giopStream* g,omni::alignment_t align,size_t sz) {
 
 	if (g->inputExpectAnotherFragment()) {
 	  // The incoming message is fragmented at the wrong boundary!!!
-	  inputTerminalProtocolError(g);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__);
 	  // never reach here
 	}
 	// Very bad. Should never happen given our invariant.
@@ -1117,7 +1169,7 @@ giopImpl12::copyInputData(giopStream* g,void* b, size_t sz,
 
       if (g->inputExpectAnotherFragment()) {
 	// The incoming message is fragmented at the wrong boundary!!!
-	inputTerminalProtocolError(g);
+	inputTerminalProtocolError(g, __FILE__, __LINE__);
 	// never reach here
       }
       // Very bad. Should never happen given our invariant.
@@ -1220,14 +1272,16 @@ giopImpl12::currentInputPtr(const giopStream* g) {
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl12::inputTerminalProtocolError(giopStream* g) {
-
+giopImpl12::inputTerminalProtocolError(giopStream* g,
+				       const char* file, int line)
+{
   // XXX We may choose to send a message error to the other end.
   if (omniORB::trace(1)) {
     omniORB::logger l;
     l << "From endpoint: " << g->pd_strand->connection->peeraddress()
-      <<". Detected GIOP 1.2 protocol error in input message. "
-      << "Connection is closed.\n";
+      << ". Detected GIOP 1.2 protocol error in input message. "
+      << file << ":" << line
+      << ". Connection is closed.\n";
   }
 
   inputRaiseCommFailure(g);
@@ -1271,6 +1325,7 @@ giopImpl12::outputNewMessage(giopStream* g) {
 			   g->pd_currentOutputBuffer->end);
   g->outputFragmentSize(0);
   g->outputMessageSize(0);
+  *((CORBA::ULong*)(hdr + 8)) = 0xffffffff;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -1306,6 +1361,7 @@ giopImpl12::outputMessageEnd(giopStream* g) {
 	  // of the header.
 	  // The header's size is stored in the buffer
 	  CORBA::ULong hdrsz = *((CORBA::ULong*)(outbuf_begin + 8));
+	  OMNIORB_ASSERT(hdrsz != 0xffffffff);
 	  if ( omni::align_to(hdrsz,omni::ALIGN_8) == (omni::ptr_arith_t)(sz+12)) {
 	    // No argument has been marshalled, remove the padding
 	    sz = hdrsz - 12;
@@ -1399,8 +1455,8 @@ giopImpl12::marshalRequestHeader(giopStream* g) {
   omniInterceptors::clientSendRequest_T::info_T info(*g,
 				                     *(giop_c.ior()),
 						     calldesc.op(),
-						     response_expected,
-						     !response_expected);
+						     !response_expected,
+						     response_expected);
   omniInterceptorP::visit(info);
 
   CORBA::Octet v = 0;
@@ -1596,6 +1652,14 @@ giopImpl12::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
   giop_s.service_contexts() >>= s;
 
   // End of reply header
+
+  if (giop_s.service_contexts().length() == 0){
+    // Set header size in buffer
+    CORBA::ULong sz = ((omni::ptr_arith_t)g->pd_outb_mkr - 
+		       (omni::ptr_arith_t)hdr - 12);
+    *((CORBA::ULong*)(hdr + 8)) = sz;
+  }
+
   s.alignOutput(omni::ALIGN_8);
 
   // RepoId
@@ -1604,10 +1668,8 @@ giopImpl12::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
 
   // system exception value
   ex.minor() >>= s;
-  CORBA::ULong(ex.completed()) >>= s;
 
-  CORBA::ULong sz = (omni::ptr_arith_t)g->pd_outb_mkr - 
-                    (omni::ptr_arith_t)hdr -12;
+  CORBA::ULong(ex.completed()) >>= s;
 
   outputMessageEnd(g);
 }
@@ -1891,7 +1953,14 @@ giopImpl12::outputFlush(giopStream* g,CORBA::Boolean knownFragmentSize) {
     avail = ((avail + 7) >> 3) << 3;
 
     omni::ptr_arith_t newmkr = (omni::ptr_arith_t) g->pd_outb_mkr + avail;
-    if (newmkr < (omni::ptr_arith_t)g->pd_outb_end) {
+
+    // If the new position is inside the buffer, set the end pointer.
+    // Note that if avail is very large, newmkr may wrap around and be
+    // < pd_outb_mkr.
+
+    if ((newmkr >= (omni::ptr_arith_t)g->pd_outb_mkr &&
+	 newmkr <  (omni::ptr_arith_t)g->pd_outb_end)) {
+
       g->pd_outb_end = (void*) newmkr;
     }
   }
