@@ -29,6 +29,70 @@
 
 /*
   $Log: giopServer.cc,v $
+  Revision 1.22.2.46  2005/11/20 15:33:03  dgrisby
+  New connection cleanup meant some connection state was not completely
+  deleted.
+
+  Revision 1.22.2.45  2005/11/17 11:35:52  dgrisby
+  In thread pool mode, bidir workers become wedged waiting for incoming
+  calls, so they must be closed in giopServer::deactivate.
+
+  Revision 1.22.2.44  2005/11/16 17:35:25  dgrisby
+  New connectionWatchPeriod and connectionWatchImmediate parameters.
+
+  Revision 1.22.2.43  2005/11/15 11:07:56  dgrisby
+  More shutdown cleanup.
+
+  Revision 1.22.2.42  2005/11/14 15:20:08  dgrisby
+  Race between giopServer destruction and closure of connections in
+  thread pool mode.
+
+  Revision 1.22.2.41  2005/11/14 10:58:23  dgrisby
+  Better connection / thread shutdown behaviour.
+
+  Revision 1.22.2.40  2005/11/04 14:26:02  dgrisby
+  Open connections would prevent shutdown on Windows.
+
+  Revision 1.22.2.39  2005/10/13 11:38:16  dgrisby
+  Dump CloseConnection messages.
+
+  Revision 1.22.2.38  2005/05/03 10:10:48  dgrisby
+  Avoid deadlock caused by trying to deactivation SSL rendezvouser more
+  than once.
+
+  Revision 1.22.2.37  2005/04/10 22:17:19  dgrisby
+  Fixes to connection management. Thanks Jon Biggar.
+
+  Revision 1.22.2.36  2005/03/10 11:28:29  dgrisby
+  Race condition between setSelectable / clearSelectable.
+
+  Revision 1.22.2.35  2005/02/23 12:27:31  dgrisby
+  Another race in setSelectable with connection shutdown. Thanks Peter
+  Klotz.
+
+  Revision 1.22.2.34  2005/02/13 20:52:51  dgrisby
+  Change threadPoolWatchConnection parameter to be an integer rather
+  than a boolean. Value is the count of threads that can be handling a
+  connection when one decides to watch it.
+
+  Revision 1.22.2.33  2004/12/20 18:17:55  dgrisby
+  Avoid race condition on connection disconnection.
+
+  Revision 1.22.2.32  2004/10/17 20:14:32  dgrisby
+  Updated support for OpenVMS. Many thanks to Bruce Visscher.
+
+  Revision 1.22.2.31  2004/09/13 10:03:13  dgrisby
+  Fix for giopServer shutdown issue. Thanks Serguei Kolos.
+
+  Revision 1.22.2.30  2004/04/08 10:02:20  dgrisby
+  In thread pool mode, close connections that will not be selectable.
+
+  Revision 1.22.2.29  2004/03/02 15:32:07  dgrisby
+  Fix locking in connection shutdown.
+
+  Revision 1.22.2.28  2004/02/27 12:40:09  dgrisby
+  Fix race conditions in connection shutdown. Thanks Serguei Kolos.
+
   Revision 1.22.2.27  2003/10/20 16:11:12  dgrisby
   Race condition in connection closure.
 
@@ -103,7 +167,6 @@
 
 */
 
-
 #include <omniORB4/CORBA.h>
 #include <invoker.h>
 #include <giopServer.h>
@@ -111,6 +174,7 @@
 #include <giopRendezvouser.h>
 #include <giopMonitor.h>
 #include <giopStrand.h>
+#include <giopStream.h>
 #include <giopStreamImpl.h>
 #include <initialiser.h>
 #include <omniORB4/omniInterceptors.h>
@@ -158,19 +222,49 @@ CORBA::ULong   orbParameters::maxServerThreadPoolSize        = 100;
 //
 //   Valid values = (n >= 1) 
 
-CORBA::Boolean orbParameters::threadPoolWatchConnection      = 1;
-//   1 means that after dispatching an upcall in thread pool mode, the
-//   thread should watch the connection for a short time before
-//   returning to the pool. This leads to less thread switching for
-//   series of calls from a single client, but is less fair if there
-//   are concurrent clients.
+CORBA::ULong   orbParameters::threadPoolWatchConnection      = 1;
+//   After dispatching an upcall in thread pool mode, the thread that
+//   has just performed the call can watch the connection for a short
+//   time before returning to the pool. This leads to less thread
+//   switching for a series of calls from a single client, but is less
+//   fair if there are concurrent clients. The connection is watched
+//   if the number of threads concurrently handling the connection is
+//   <= the value of this parameter. i.e. if the parameter is zero,
+//   the connection is never watched; if it is 1, the last thread
+//   managing a connection watches it; if 2, the connection is still
+//   watched if there is one other thread still in an upcall for the
+//   connection, and so on.
 //
-//  Valid values = 0 or 1
+//  Valid values = (n >= 0)
+
+CORBA::Boolean orbParameters::connectionWatchImmediate      = 0;
+//   When a thread handles an incoming call, it unmarshals the
+//   arguments then marks the connection as watchable by the connection
+//   watching thread, in case the client sends a concurrent call on the
+//   same connection. If this parameter is set to the default false,
+//   the connection is not actually watched until the next connection
+//   watch period (determined by the connectionWatchPeriod parameter).
+//   If connectionWatchImmediate is set true, the connection watching
+//   thread is immediately signalled to watch the connection. That
+//   leads to faster interactive response to clients that multiplex
+//   calls, but adds significant overhead along the call chain.
+//
+//   Note that this setting has no effect on Windows, since it has no
+//   mechanism for signalling the connection watching thread.
+
+
+////////////////////////////////////////////////////////////////////////////
+static const char* plural(CORBA::ULong val)
+{
+  return val == 1 ? "" : "s";
+}
 
 
 ////////////////////////////////////////////////////////////////////////////
 giopServer::giopServer() : pd_state(IDLE), pd_nconnections(0),
-			   pd_cond(&pd_lock), pd_n_temporary_workers(0)
+			   pd_cond(&pd_lock),
+			   pd_n_temporary_workers(0),
+			   pd_n_dedicated_workers(0)
 {
   pd_thread_per_connection = orbParameters::threadPerConnectionPolicy;
   pd_connectionState = new connectionState*[connectionState::hashsize];
@@ -420,6 +514,7 @@ giopServer::activate()
       }
       task->insert(cs->workers);
       cs->connection->pd_n_workers++;
+      pd_n_dedicated_workers++;
     }
     else {
       pd_lock.unlock();
@@ -454,6 +549,29 @@ giopServer::activate()
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+static inline void
+sendCloseConnection(giopStrand* s)
+{
+  // Send close connection message.
+  char hdr[12];
+  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
+  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
+  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
+  hdr[7] = (char)GIOP::CloseConnection;
+  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
+
+  if (omniORB::trace(25)) {
+    omniORB::logger log;
+    log << "sendCloseConnection: to " << s->connection->peeraddress()
+	<< " 12 bytes\n";
+  }
+  if (omniORB::trace(30))
+    giopStream::dumpbuf((unsigned char*)hdr, 12);
+
+  s->connection->Send(hdr,12);
+}
+
 ////////////////////////////////////////////////////////////////////////////
 void
 giopServer::deactivate()
@@ -464,51 +582,174 @@ giopServer::deactivate()
 
   pd_state = INFLUX;
 
- again:
-  CORBA::Boolean waitforcompletion = 0;
-  {
+  CORBA::Boolean waitforcompletion;
+  CORBA::Boolean rendezvousers_terminated = 0;
+
+  omniORB::logs(25, "giopServer deactivate...");
+
+  unsigned long s, ns;
+
+  unsigned long timeout;
+  if (orbParameters::scanGranularity)
+    timeout = orbParameters::scanGranularity;
+  else
+    timeout = 5;
+
+  if (pd_nconnections) {
+    omniORB::logs(25, "Close connections with threads and monitors...");
+
+    CORBA::ULong closed_count = 0;
+
     for (CORBA::ULong i=0; i < connectionState::hashsize; i++) {
       connectionState** head = &(pd_connectionState[i]);
       while (*head) {
-	{
-	  // Send close connection message.
-	  GIOP::Version ver = giopStreamImpl::maxVersion()->version();
-	  char hdr[12];
-	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = ver.major;   hdr[5] = ver.minor;
-	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
-	  hdr[7] = (char)GIOP::CloseConnection;
-	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
-	  (*head)->connection->Send(hdr,12);
+	if ((*head)->connection->pd_has_dedicated_thread ||
+	    (*head)->strand->biDir) {
+
+	  sendCloseConnection((*head)->strand);
+	  (*head)->connection->Shutdown();
+	  ++closed_count;
 	}
-	(*head)->connection->Shutdown();
 	head = &((*head)->next);
       }
     }
-    if (pd_nconnections) waitforcompletion = 1;
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      l << "Closed " << closed_count << " connection" << plural(closed_count)
+	<< " out of " << pd_nconnections << ".\n";
+    }
+
+    if (pd_n_dedicated_workers) {
+      if (omniORB::trace(25)) {
+	omniORB::logger l;
+	l << "Wait for " << pd_n_dedicated_workers
+	  << " dedicated thread" << plural(pd_n_dedicated_workers)
+	  << " to finish...\n";
+      }
+
+      omni_thread::get_time(&s, &ns, timeout);
+
+      int go = 1;
+      while (go && pd_n_dedicated_workers) {
+	go = pd_cond.timedwait(s, ns);
+      }
+      if (omniORB::trace(25)) {
+	omniORB::logger l;
+	if (!go)
+	  l << "Timed out. ";
+
+	l << pd_nconnections << " connection" << plural(pd_nconnections)
+	  << " and " << pd_n_dedicated_workers << " dedicated worker"
+	  << plural(pd_n_dedicated_workers) << " remaining.\n";
+      }
+    }
   }
-  {
-    Link* p = pd_rendezvousers.next;
-    if (p != &pd_rendezvousers) waitforcompletion = 1;
+
+  // Stop rendezvousers
+  Link* p = pd_rendezvousers.next;
+  if (p != &pd_rendezvousers) {
+    omniORB::logs(25, "Terminate rendezvousers...");
 
     for (; p != &pd_rendezvousers; p = p->next) {
       ((giopRendezvouser*)p)->terminate();
     }
+
+    omni_thread::get_time(&s, &ns, timeout);
+    int go = 1;
+    while (go && pd_rendezvousers.next != & pd_rendezvousers) {
+      go = pd_cond.timedwait(s, ns);
+    }
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      if (go)
+	l << "Rendezvousers terminated.\n";
+      else
+	l << "Timed out waiting for rendezvousers to terminate.\n";
+    }
   }
 
+  // Stop bidir monitors
   if (!Link::is_empty(pd_bidir_monitors)) {
-    waitforcompletion = 1;
+    omniORB::logs(25, "Deactivate bidirectional monitors...");
 
     Link* m = pd_bidir_monitors.next;
     for (; m != &pd_bidir_monitors; m = m->next) {
       ((giopMonitor*)m)->deactivate();
     }
+
+    omni_thread::get_time(&s, &ns, timeout);
+    int go = 1;
+    while (go && !Link::is_empty(pd_bidir_monitors)) {
+      go = pd_cond.timedwait(s, ns);
+    }
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      if (go)
+	l << "Monitors deactivated.\n";
+      else
+	l << "Timed out waiting for monitors to deactivate.\n";
+    }
   }
 
+  if (pd_nconnections) {
+    omniORB::logs(25, "Close remaining connections...");
+    CORBA::ULong closed_count = 0;
+
+    for (CORBA::ULong i=0; i < connectionState::hashsize; i++) {
+      connectionState** head = &(pd_connectionState[i]);
+      while (*head) {
+	if (!((*head)->connection->pd_has_dedicated_thread ||
+	      (*head)->strand->biDir)) {
+
+	  sendCloseConnection((*head)->strand);
+	  (*head)->connection->Shutdown();
+
+	  // Start a worker to notice the connection closure.
+	  giopWorker* task = new giopWorker((*head)->strand, this, 1);
+	  orbAsyncInvoker->insert(task);
+	  task->insert((*head)->workers);
+	  ++(*head)->connection->pd_n_workers;
+	  ++pd_n_temporary_workers;
+	  ++closed_count;
+	}
+	head = &((*head)->next);
+      }
+    }
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      l << "Closed " << closed_count << " connection" << plural(closed_count)
+	<< " out of " << pd_nconnections << ".\n";
+    }
+  }
+
+  if (pd_n_temporary_workers) {
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      l << "Wait for " << pd_n_temporary_workers << " temporary worker"
+	<< plural(pd_n_temporary_workers) << "...\n";
+    }
+    int go = 1;
+    while (go && pd_n_temporary_workers) {
+      go = pd_cond.timedwait(s, ns);
+    }
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      if (!go)
+	l << "Timed out. ";
+      l << pd_n_temporary_workers << " temporary worker"
+	<< plural(pd_n_temporary_workers) << " remaining.\n";
+    }
+  }
+
+  // Close bidir connections
+  if (pd_bidir_strands.size()) {
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      l << "Close " << pd_bidir_strands.size()
+	<< " bidirectional connections...\n";
+    }
 
-  {
     omnivector<giopStrand*>::iterator i = pd_bidir_strands.begin();
-
     while (i != pd_bidir_strands.end()) {
       giopStrand* g = *i;
       pd_bidir_strands.erase(i);
@@ -516,21 +757,9 @@ giopServer::deactivate()
       g->deleteStrandAndConnection();
     }
   }
-
-  if (waitforcompletion) {
-    // Here we relinquish the mutex to give the rendezvousers and workers
-    // a chance to remove themselves from the lists. Notice that the server
-    // is now in the INFLUX state. This means that no start, stop, remove,
-    // instantiate can progress until we are done here.
-    omniORB::logs(25, "giopServer waits for completion of rendezvousers "
-		  "and workers");
-    pd_cond.wait();
-    omniORB::logs(25, "giopServer back from waiting.");
-    goto again;
-  }
-  else {
-    pd_state = IDLE;
-  }
+  pd_state = IDLE;
+  pd_cond.broadcast();
+  omniORB::logs(25, "giopServer deactivated.");
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -628,6 +857,7 @@ giopServer::csInsert(giopConnection* conn)
 
 
   giopStrand* s = new giopStrand(conn,this);
+  s->version.major = 1; s->version.minor = 0;
   {
     ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,0);
     omni_tracedmutex_lock sync(*omniTransportLock);
@@ -710,8 +940,7 @@ giopServer::notifyRzNewConnection(giopRendezvouser* r, giopConnection* conn)
       connectionState* cs = csInsert(conn);
 
       if (conn->pd_has_dedicated_thread) {
-	giopWorker* task = new giopWorker(cs->strand,this,
-					  !conn->pd_has_dedicated_thread);
+	giopWorker* task = new giopWorker(cs->strand, this, 0);
 	if (!orbAsyncInvoker->insert(task)) {
 	  // Cannot start serving this new connection.
 	  if (omniORB::trace(1)) {
@@ -736,8 +965,25 @@ giopServer::notifyRzNewConnection(giopRendezvouser* r, giopConnection* conn)
 	}
 	task->insert(cs->workers);
 	conn->pd_n_workers++;
+	pd_n_dedicated_workers++;
       }
       else {
+	if (!conn->isSelectable()) {
+	  if (omniORB::trace(20)) {
+	    omniORB::logger log;
+	    log << "Connection from " << conn->peeraddress()
+		<< " is not selectable. Closing it.\n";
+	  }
+	  {
+	    omni_tracedmutex_lock sync(*omniTransportLock);
+	    cs->strand->safeDelete();
+	  }
+	  csRemove(conn);
+	  pd_lock.unlock();
+	  delete cs;
+	  pd_lock.lock();
+	  throw outOfResource();
+	}
 	pd_lock.unlock();
 	conn->setSelectable(1);
 	pd_lock.lock();
@@ -745,6 +991,11 @@ giopServer::notifyRzNewConnection(giopRendezvouser* r, giopConnection* conn)
       break;
     }
   default:
+    if (omniORB::trace(25)) {
+      omniORB::logger l;
+      l << "giopServer terminate connection from "
+	<< conn->peeraddress() << ".\n";
+    }
     throw Terminate();
   }
 }
@@ -767,6 +1018,11 @@ giopServer::notifyRzDone(giopRendezvouser* r, CORBA::Boolean exit_on_error)
 
   giopEndpoint* ept = r->endpoint();
 
+  if (omniORB::trace(25)) {
+    omniORB::logger l;
+    l << "giopRendezvouser for endpoint " << ept->address() << " exit.\n";
+  }
+
   r->remove();
 
   delete r;
@@ -787,9 +1043,7 @@ giopServer::notifyRzDone(giopRendezvouser* r, CORBA::Boolean exit_on_error)
   }
 
   if (pd_state == INFLUX) {
-    if (Link::is_empty(pd_rendezvousers) &&
-	pd_nconnections == 0             &&
-	Link::is_empty(pd_bidir_monitors)   ) {
+    if (Link::is_empty(pd_rendezvousers)) {
       pd_cond.broadcast();
     }
   }
@@ -857,7 +1111,7 @@ giopServer::removeConnectionAndWorker(giopWorker* w)
     giopConnection* conn = w->strand()->connection;
 
     conn->pd_dying = 1; // From now on, the giopServer will not create
-    // any more workers to serve this connection.
+                        // any more workers to serve this connection.
 
     cs = csLocate(conn);
 
@@ -874,7 +1128,12 @@ giopServer::removeConnectionAndWorker(giopWorker* w)
     // is therefore safe to delete this record.
     pd_lock.lock();
 
-    if (w->singleshot()) pd_n_temporary_workers--;
+    int workers;
+
+    if (w->singleshot())
+      workers = --pd_n_temporary_workers;
+    else
+      workers = --pd_n_dedicated_workers;
 
     w->remove();
     delete w;
@@ -887,9 +1146,7 @@ giopServer::removeConnectionAndWorker(giopWorker* w)
     }
 
     if (pd_state == INFLUX) {
-      if (Link::is_empty(pd_rendezvousers) &&
-	  pd_nconnections == 0             &&
-	  Link::is_empty(pd_bidir_monitors)   ) {
+      if (workers == 0) {
 	pd_cond.broadcast();
       }
     }
@@ -950,6 +1207,8 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
       delete w;
       conn->pd_n_workers--;
       pd_n_temporary_workers--;
+      if (pd_state == INFLUX && pd_n_temporary_workers == 0)
+	pd_cond.broadcast();
       return 0;
     }
   }
@@ -972,24 +1231,14 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
       // If there are other workers for this connection, or there are
       // too many temporary workers, let this worker finish.
 
-      if (conn->pd_n_workers > 1 ||
+      if (conn->pd_n_workers > orbParameters::threadPoolWatchConnection ||
 	  pd_n_temporary_workers > orbParameters::maxServerThreadPoolSize) {
 
-	w->remove();
-	delete w;
-	conn->pd_n_workers--;
-	pd_n_temporary_workers--;
-
 	select_and_return = 1;
       }
     }
-    if (select_and_return) {
-      // Connection is selectable now
-      conn->setSelectable(1);
-      return 0;
-    }
 
-    if (orbParameters::threadPoolWatchConnection) {
+    if (!select_and_return) {
       // Call Peek(). This thread will be used for a short time to
       // monitor the connection. If the connection is available for
       // reading, the callback function peekCallBack is called. We can
@@ -1004,29 +1253,33 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
       }
     }
 
-    // Connection is selectable now
-    conn->setSelectable(1);
+    // If connection is selectable, it's selectable now
+    if (!conn->pd_dying)
+      conn->setSelectable(2);
 
     // Worker is no longer needed.
     {
       omni_tracedmutex_lock sync(pd_lock);
 
-      if (conn->pd_n_workers == 1 && conn->pd_dying) {
-	// Connection is dying. Go round again so this thread spots
-	// the condition.
-	omniORB::logs(25, "Last worker sees connection is dying.");
-	return 1;
-      }
       w->remove();
       delete w;
       conn->pd_n_workers--;
       pd_n_temporary_workers--;
+
+      if (pd_state == INFLUX) {
+	omniORB::logs(25, "Temporary worker finishing.");
+	if (pd_n_temporary_workers == 0)
+	  pd_cond.broadcast();
+      }
     }
     return 0;
   }
-  // Never reach here
+#ifndef __DECCXX
+  // Never reach here (and the hp/compaq/dec compiler is smart enough
+  // to figure that out).
   OMNIORB_ASSERT(0);
   return 0;
+#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -1061,7 +1314,8 @@ giopServer::notifyWkPreUpCall(giopWorker* w, CORBA::Boolean data_in_buffer) {
 	// If only one thread per connection is allowed, there is no
 	// need to setSelectable, since we won't be able to act on any
 	// interleaved calls that arrive.
-	conn->setSelectable(0,data_in_buffer);
+	conn->setSelectable(orbParameters::connectionWatchImmediate,
+			    data_in_buffer);
       }
     }
     else {
@@ -1073,13 +1327,15 @@ giopServer::notifyWkPreUpCall(giopWorker* w, CORBA::Boolean data_in_buffer) {
 	n = conn->pd_dedicated_thread_in_upcall;
       }
       if (n) {
-	conn->setSelectable(0,data_in_buffer);
+	conn->setSelectable(orbParameters::connectionWatchImmediate,
+			    data_in_buffer);
       }
     }
   }
   else {
     // This connection is managed with the thread-pool policy
-    conn->setSelectable(0,data_in_buffer);
+    conn->setSelectable(orbParameters::connectionWatchImmediate,
+			data_in_buffer);
   }
 }
 
@@ -1112,9 +1368,7 @@ giopServer::notifyMrDone(giopMonitor* m, CORBA::Boolean exit_on_error)
   m->remove();
   delete m;
   if (pd_state == INFLUX) {
-    if (Link::is_empty(pd_rendezvousers) &&
-	pd_nconnections == 0             &&
-	Link::is_empty(pd_bidir_monitors)   ) {
+    if (Link::is_empty(pd_bidir_monitors)) {
       pd_cond.broadcast();
     }
   }
@@ -1318,9 +1572,38 @@ public:
 
   threadPoolWatchConnectionHandler() : 
     orbOptions::Handler("threadPoolWatchConnection",
-			"threadPoolWatchConnection = 0 or 1",
+			"threadPoolWatchConnection = n >= 0",
 			1,
-			"-ORBthreadPoolWatchConnection < 0 | 1 >") {}
+			"-ORBthreadPoolWatchConnection < n >= 0 >") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+
+    CORBA::ULong v;
+    if (!orbOptions::getULong(value,v)) {
+      throw orbOptions::BadParam(key(),value,
+				 orbOptions::expect_ulong_msg);
+    }
+    orbParameters::threadPoolWatchConnection = v;
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    orbOptions::addKVULong(key(),orbParameters::threadPoolWatchConnection,
+			   result);
+  }
+};
+
+static threadPoolWatchConnectionHandler threadPoolWatchConnectionHandler_;
+
+/////////////////////////////////////////////////////////////////////////////
+class connectionWatchImmediateHandler : public orbOptions::Handler {
+public:
+
+  connectionWatchImmediateHandler() : 
+    orbOptions::Handler("connectionWatchImmediate",
+			"connectionWatchImmediate = 0 or 1",
+			1,
+			"-ORBconnectionWatchImmediate < 0 | 1 >") {}
 
 
   void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
@@ -1330,16 +1613,18 @@ public:
       throw orbOptions::BadParam(key(),value,
 				 orbOptions::expect_boolean_msg);
     }
-    orbParameters::threadPoolWatchConnection = v;
+    orbParameters::connectionWatchImmediate = v;
   }
 
   void dump(orbOptions::sequenceString& result) {
-    orbOptions::addKVBoolean(key(),orbParameters::threadPoolWatchConnection,
+    orbOptions::addKVBoolean(key(),orbParameters::connectionWatchImmediate,
 			     result);
   }
 };
 
-static threadPoolWatchConnectionHandler threadPoolWatchConnectionHandler_;
+static connectionWatchImmediateHandler connectionWatchImmediateHandler_;
+
+
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1356,6 +1641,7 @@ public:
     orbOptions::singleton().registerHandler(maxServerThreadPerConnectionHandler_);
     orbOptions::singleton().registerHandler(maxServerThreadPoolSizeHandler_);
     orbOptions::singleton().registerHandler(threadPoolWatchConnectionHandler_);
+    orbOptions::singleton().registerHandler(connectionWatchImmediateHandler_);
   }
 
   void attach() {
@@ -1462,7 +1748,7 @@ OMNI_NAMESPACE_END(omni)
 //        - setSelectable
 //
 //     notifyWkDone()
-//        - setSelectable(1) if n worker == 1.
+//        - setSelectable(2) if n worker == 1.
 //
 
 ////////////////////////////////////////////////////////////////////////////
