@@ -28,6 +28,28 @@
 
 /*
   $Log: giopStrand.cc,v $
+  Revision 1.1.4.25  2005/11/14 10:58:23  dgrisby
+  Better connection / thread shutdown behaviour.
+
+  Revision 1.1.4.24  2005/10/13 11:38:16  dgrisby
+  Dump CloseConnection messages.
+
+  Revision 1.1.4.23  2005/04/10 22:17:18  dgrisby
+  Fixes to connection management. Thanks Jon Biggar.
+
+  Revision 1.1.4.22  2005/03/16 09:16:59  dgrisby
+  Previous change to CancelRequest handling accidentally broke
+  server-side connection shutdown.
+
+  Revision 1.1.4.21  2004/10/17 21:48:40  dgrisby
+  Support CancelRequest better.
+
+  Revision 1.1.4.20  2004/03/30 14:26:36  dgrisby
+  Better fix for scavenger thread restarting after shutdown.
+
+  Revision 1.1.4.19  2004/03/30 13:14:47  dgrisby
+  Allow scavenger thread to restart.
+
   Revision 1.1.4.18  2002/09/04 23:29:30  dgrisby
   Avoid memory corruption with multiple list removals.
 
@@ -157,6 +179,30 @@ private:
   void removeIdle(StrandList& src,StrandList& dest, CORBA::Boolean skip_bidir);
 };
 
+////////////////////////////////////////////////////////////////////////
+static inline void
+sendCloseConnection(giopStrand* s)
+{
+  // Send close connection message.
+  char hdr[12];
+  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
+  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
+  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
+  hdr[7] = (char)GIOP::CloseConnection;
+  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
+
+  if (omniORB::trace(25)) {
+    omniORB::logger log;
+    log << "sendCloseConnection: to " << s->connection->peeraddress()
+	<< " 12 bytes\n";
+  }
+  if (omniORB::trace(30))
+    giopStream::dumpbuf((unsigned char*)hdr, 12);
+
+  s->connection->Send(hdr,12);
+}
+
+
 ////////////////////////////////////////////////////////////////////////
 giopStrand::giopStrand(const giopAddress* addr) :
   pd_safelyDeleted(0),
@@ -458,7 +504,10 @@ giopStrand::acquireServer(giopWorker* w)
 
   sp->state(IOP_S::Idle);
   if (!sp->impl()) {
-    sp->impl(giopStreamImpl::maxVersion());
+    giopStreamImpl *impl = giopStreamImpl::maxVersion();
+    sp->impl(impl);
+    if (version.major == 0)
+	version = impl->version();
   }
   // the codeset convertors are filled in by the codeset interceptor
   // before a request is unmarshalled.
@@ -531,7 +580,12 @@ giopStrand::releaseServer(IOP_S* iop_s)
     giop_s->giopStreamList::insert(servers);
   }
 
-  if (remove) delete giop_s;
+  if (remove) {
+    if (giop_s->state() != IOP_S::WaitingForReply)
+      delete giop_s;
+    else
+      restart_idle = 0;
+  }
 
   if (restart_idle && !biDir) {
     CORBA::Boolean success = startIdleCounter();
@@ -572,10 +626,10 @@ CORBA::Boolean
 giopStrand::startIdleCounter() {
   ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,1);
 
-  if (idlebeats >= 0) 
+  if (idlebeats >= 0) {
     // The idle counter is already active or has already expired.
     return 0;
-  
+  }
   if (isClient()) {
     idlebeats = (idleOutgoingBeats) ? (CORBA::Long)idleOutgoingBeats : -1;
   }
@@ -590,10 +644,10 @@ CORBA::Boolean
 giopStrand::stopIdleCounter() {
   ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,1);
 
-  if (idlebeats == 0)
+  if (idlebeats == 0) {
     // The idle counter has already expired.
     return 0;
-
+  }
   idlebeats = -1;
   return 1;
 }
@@ -685,13 +739,7 @@ Scavenger::execute()
 	if ( s->version.minor >= 2 ) {
 	  // GIOP 1.2 or above requires the client send a closeconnection
 	  // message.
-	  char hdr[12];
-	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
-	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
-	  hdr[7] = (char)GIOP::CloseConnection;
-	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
-	  s->connection->Send(hdr,12);
+	  sendCloseConnection(s);
 	}
 	s->safeDelete(1);
       }
@@ -713,17 +761,7 @@ Scavenger::execute()
 	  log << "Scavenger close connection from " 
 	      << s->connection->peeraddress() << "\n";
 	}	
-	{
-	  // Send close connection message.
-	  GIOP::Version ver = giopStreamImpl::maxVersion()->version();
-	  char hdr[12];
-	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = ver.major;   hdr[5] = ver.minor;
-	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
-	  hdr[7] = (char)GIOP::CloseConnection;
-	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
-	  s->connection->Send(hdr,12);
-	}
+	sendCloseConnection(s);
 	s->connection->Shutdown();
       }
     }
@@ -782,6 +820,7 @@ Scavenger::terminate()
 void
 Scavenger::initialise()
 {
+  Scavenger::shutdown = 0;
   Scavenger::mutex = new omni_tracedmutex();
   Scavenger::cond  = new omni_tracedcondition(Scavenger::mutex);
 }
@@ -922,10 +961,13 @@ public:
     Scavenger::initialise();
   }
   void detach() {
+    omniORB::logs(25, "Terminate strand scavenger.");
     Scavenger::terminate();
 
     omni_tracedmutex_lock sync(*omniTransportLock);
 
+    omniORB::logs(25, "Close remaining strands.");
+
     // Close client strands
     {
       StrandList* p = giopStrand::active_timedout.next;
@@ -935,7 +977,7 @@ public:
 	s->StrandList::remove();
 	s->RopeLink::remove();
 	s->state(giopStrand::DYING);
-	if (omniORB::trace(30)) {
+	if (omniORB::trace(25)) {
 	  omniORB::logger log;
 	  log << "Shutdown close connection to "
 	      << s->address->address() << "\n";
@@ -943,13 +985,7 @@ public:
 	if ( s->version.minor >= 2 ) {
 	  // GIOP 1.2 or above requires the client send a closeconnection
 	  // message.
-	  char hdr[12];
-	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
-	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
-	  hdr[7] = (char)GIOP::CloseConnection;
-	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
-	  s->connection->Send(hdr,12);
+	  sendCloseConnection(s);
 	}
 	s->safeDelete(1);
       }
@@ -962,7 +998,7 @@ public:
 	s->StrandList::remove();
 	s->RopeLink::remove();
 	s->state(giopStrand::DYING);
-	if (omniORB::trace(30)) {
+	if (omniORB::trace(25)) {
 	  omniORB::logger log;
 	  log << "Shutdown close connection to "
 	      << s->address->address() << "\n";
@@ -970,13 +1006,7 @@ public:
 	if ( s->version.minor >= 2 ) {
 	  // GIOP 1.2 or above requires the client send a closeconnection
 	  // message.
-	  char hdr[12];
-	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
-	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
-	  hdr[7] = (char)GIOP::CloseConnection;
-	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
-	  s->connection->Send(hdr,12);
+	  sendCloseConnection(s);
 	}
 	s->safeDelete(1);
       }
@@ -990,22 +1020,12 @@ public:
 	s->StrandList::remove();
 	s->RopeLink::remove();
 	s->state(giopStrand::DYING);
-	if (omniORB::trace(30)) {
+	if (omniORB::trace(25)) {
 	  omniORB::logger log;
 	  log << "Shutdown close connection from " 
 	      << s->connection->peeraddress() << "\n";
 	}	
-	{
-	  // Send close connection message.
-	  GIOP::Version ver = giopStreamImpl::maxVersion()->version();
-	  char hdr[12];
-	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = ver.major;   hdr[5] = ver.minor;
-	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
-	  hdr[7] = (char)GIOP::CloseConnection;
-	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
-	  s->connection->Send(hdr,12);
-	}
+	sendCloseConnection(s);
 	s->connection->Shutdown();
       }
     }
