@@ -28,6 +28,15 @@
 
 /*
   $Log: giopStream.cc,v $
+  Revision 1.1.4.30  2005/10/13 11:38:16  dgrisby
+  Dump CloseConnection messages.
+
+  Revision 1.1.4.29  2005/03/03 12:55:55  dgrisby
+  Minor log output clean-up. Thanks Peter Klotz.
+
+  Revision 1.1.4.28  2004/07/23 13:25:44  dgrisby
+  New traceExceptions option.
+
   Revision 1.1.4.27  2003/09/25 13:47:44  dgrisby
   Log connection attempts.
 
@@ -123,8 +132,6 @@
 
 OMNI_NAMESPACE_BEGIN(omni)
 
-static void dumpbuf(unsigned char* buf, size_t sz);
-
 ////////////////////////////////////////////////////////////////////////
 CORBA::ULong giopStream::directSendCutOff = 16384;
 CORBA::ULong giopStream::directReceiveCutOff = 1024;
@@ -516,7 +523,7 @@ giopStream::CommFailure::_raise(CORBA::ULong minor,
 {
   if (status != CORBA::COMPLETED_NO) retry = 0;
 #ifndef OMNIORB_NO_EXCEPTION_LOGGING
-  if( omniORB::trace(10) ) {
+  if( omniORB::traceExceptions ) {
     omniORB::logger l;
     l << "throw giopStream::CommFailure from "
       << omniExHelper::strip(filename) 	
@@ -1190,7 +1197,8 @@ static inline char printable_char(char c) {
 }
 
 /////////////////////////////////////////////////////////////////////////
-static void dumpbuf(unsigned char* buf, size_t sz)
+void 
+giopStream::dumpbuf(unsigned char* buf, size_t sz)
 {
   static omni_tracedmutex lock;
   omni_tracedmutex_lock sync(lock);
@@ -1200,7 +1208,7 @@ static void dumpbuf(unsigned char* buf, size_t sz)
 
   const size_t dumplimit = 128;
   if (!omniORB::trace(40) && sz > dumplimit) {
-    l << dumplimit << "bytes out of " << sz << "\n";
+    l << dumplimit << " bytes out of " << sz << "\n";
     sz = dumplimit;
   }
   else {
