@@ -29,6 +29,31 @@
 
 /*
   $Log: omniInternal.cc,v $
+  Revision 1.2.2.41  2005/12/01 23:14:24  dgrisby
+  Remove unsafe log message from native exception handler.
+
+  Revision 1.2.2.40  2005/11/28 14:57:51  dgrisby
+  New abortOnNativeException parameter for Windows.
+
+  Revision 1.2.2.39  2005/09/19 14:23:56  dgrisby
+  New traceFile configuration parameter.
+
+  Revision 1.2.2.38  2005/04/08 00:06:14  dgrisby
+  Remove all remaining uses of logf.
+
+  Revision 1.2.2.37  2005/03/29 14:41:33  dgrisby
+  Use omniORB logger instead of logf. May avoid a compiler / platform
+  bug on VxWorks.
+
+  Revision 1.2.2.36  2004/10/17 20:14:33  dgrisby
+  Updated support for OpenVMS. Many thanks to Bruce Visscher.
+
+  Revision 1.2.2.35  2004/07/23 13:25:44  dgrisby
+  New traceExceptions option.
+
+  Revision 1.2.2.34  2004/03/02 15:31:22  dgrisby
+  Support for persistent server identifier.
+
   Revision 1.2.2.33  2003/08/06 20:36:49  dgrisby
   Shorten assertion failure message.
 
@@ -219,6 +244,10 @@
 #include <omniORB4/objTracker.h>
 #include <corbaOrb.h>
 
+#ifdef __WIN32__
+#  include <eh.h>
+#endif
+
 ////////////////////////////////////////////////////////////////////////////
 //             Configuration options                                      //
 ////////////////////////////////////////////////////////////////////////////
@@ -235,6 +264,11 @@ CORBA::ULong  omniORB::traceLevel = 1;
 //
 //    Valid values = (n >= 0)
 
+CORBA::Boolean  omniORB::traceExceptions = 0;
+//    If true, then system exceptions are logged when they are thrown.
+//
+//    Valid values = 0 or 1
+
 CORBA::Boolean  omniORB::traceInvocations = 0;
 //    If true, then each local and remote invocation will generate a trace 
 //    message.
@@ -285,6 +319,14 @@ CORBA::Boolean orbParameters::abortOnInternalError = 0;
 //
 //  Valid values = 0 or 1
 
+static CORBA::Boolean abortOnNativeException = 0;
+//  On Windows, "native" exceptions such as segmentation faults and
+//  divide by zero appear as C++ exceptions that can be caught with
+//  catch (...). Setting this parameter to true causes such exceptions
+//  to abort the process instead.
+//
+//  Valid values = 0 or 1
+
 
 ////////////////////////////////////////////////////////////////////////////
 
@@ -368,10 +410,10 @@ omniObjTable::resize()
   }
   CORBA::ULong newsize = newsizei;
 
-  if( omniORB::trace(15) ) {
-    omniORB::logf("Object table resizing from %lu to %lu",
-		  (unsigned long) objectTableSize,
-		  (unsigned long) newsize);
+  if (omniORB::trace(15)) {
+    omniORB::logger l;
+    l << "Object table resizing from " << objectTableSize
+      << " to " << newsize << "\n";
   }
 
   // Create and initialise new object table.
@@ -472,8 +514,10 @@ omni::releaseObjRef(omniObjRef* objref)
     objref->_setIdentity(0);
   }
 
-  if( omniORB::trace(15) )
-    omniORB::logf("ObjRef(%s) -- deleted.", objref->_mostDerivedRepoId());
+  if( omniORB::trace(15) ) {
+    omniORB::logger l;
+    l << "ObjRef(" << objref->_mostDerivedRepoId() << ") -- deleted.\n";
+  }
 
   // Destroy the reference.
   delete objref;
@@ -816,9 +860,9 @@ omni::createIdentity(omniIOR* ior, const char* target, CORBA::Boolean locked)
   omniIOR_var holder(ior); // Place the ior inside a var. If ever
                            // any function we called results in an
                            // exception being thrown, the ior is released
-                           // by the var, hence fullifilling the semantics
+                           // by the var, hence fullfilling the semantics
                            // of this function.
-                           // If this function completed normally, make
+                           // If this function completes normally, make
                            // sure that _retn() is called on the var so
                            // that the ior is not released incorrectly!
 
@@ -1015,6 +1059,35 @@ omni::createObjRef(const char* targetRepoId,
     id->gainRef(objref);
   }
 
+  if (orbParameters::persistentId.length()) {
+    // Check to see if we need to re-write the IOR.
+
+    omniIOR::IORExtraInfoList& extra = ior->getIORInfo()->extraInfo();
+
+    for (CORBA::ULong index = 0; index < extra.length(); index++) {
+
+      if (extra[index]->compid == IOP::TAG_OMNIORB_PERSISTENT_ID) {
+
+	if (!id->inThisAddressSpace()) {
+
+	  omniORB::logs(15, "Re-write local persistent object reference.");
+
+	  omniObjRef* new_objref;
+	  {
+	    omni_optional_lock sync(*internalLock, locked, locked);
+
+	    omniIOR* new_ior = new omniIOR(ior->repositoryID(),
+					   id->key(), id->keysize());
+
+	    new_objref = createObjRef(targetRepoId, new_ior, 1, 0);
+	  }
+	  releaseObjRef(objref);
+	  objref = new_objref;
+	}
+	break;
+      }
+    }
+  }
   return objref;
 }
 
@@ -1250,6 +1323,8 @@ public:
 				 orbOptions::expect_ulong_msg);
     }
     omniORB::traceLevel = v;
+    if (v >= 10)
+      omniORB::traceExceptions = 1;
   }
 
   void dump(orbOptions::sequenceString& result) {
@@ -1261,6 +1336,35 @@ public:
 
 static traceLevelHandler traceLevelHandler_;
 
+/////////////////////////////////////////////////////////////////////////////
+class traceExceptionsHandler : public orbOptions::Handler {
+public:
+
+  traceExceptionsHandler() : 
+    orbOptions::Handler("traceExceptions",
+			"traceExceptions = 0 or 1",
+			1,
+			"-ORBtraceExceptions < 0 | 1 >") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+
+    CORBA::Boolean v;
+    if (!orbOptions::getBoolean(value,v)) {
+      throw orbOptions::BadParam(key(),value,
+				 orbOptions::expect_boolean_msg);
+    }
+    omniORB::traceExceptions = v;
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    orbOptions::addKVBoolean(key(),omniORB::traceExceptions,
+			     result);
+  }
+};
+
+static traceExceptionsHandler traceExceptionsHandler_;
+
 /////////////////////////////////////////////////////////////////////////////
 class traceInvocationsHandler : public orbOptions::Handler {
 public:
@@ -1319,6 +1423,29 @@ public:
 
 static traceThreadIdHandler traceThreadIdHandler_;
 
+/////////////////////////////////////////////////////////////////////////////
+class traceFileHandler : public orbOptions::Handler {
+public:
+
+  traceFileHandler() :
+    orbOptions::Handler("traceFile",
+			"traceFile = <filename>",
+			1,
+			"-ORBtraceFile <filename>") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+    omniORB::setLogFilename(value);
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    const char* n = omniORB::getLogFilename();
+    orbOptions::addKVString(key(), n ? n : "[stderr]", result);
+  }
+};
+
+static traceFileHandler traceFileHandler_;
+
 /////////////////////////////////////////////////////////////////////////////
 class objectTableSizeHandler : public orbOptions::Handler {
 public:
@@ -1376,6 +1503,49 @@ public:
 
 static abortOnInternalErrorHandler abortOnInternalErrorHandler_;
 
+
+/////////////////////////////////////////////////////////////////////////////
+class abortOnNativeExceptionHandler : public orbOptions::Handler {
+public:
+
+  abortOnNativeExceptionHandler() : 
+    orbOptions::Handler("abortOnNativeException",
+			"abortOnNativeException = 0 or 1",
+			1,
+			"-ORBabortOnNativeException < 0 | 1 >") {}
+
+
+  void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
+
+    CORBA::Boolean v;
+    if (!orbOptions::getBoolean(value,v)) {
+      throw orbOptions::BadParam(key(),value,
+				 orbOptions::expect_boolean_msg);
+    }
+    abortOnNativeException = v;
+  }
+
+  void dump(orbOptions::sequenceString& result) {
+    orbOptions::addKVBoolean(key(),abortOnNativeException,result);
+  }
+};
+
+static abortOnNativeExceptionHandler abortOnNativeExceptionHandler_;
+
+
+#ifdef __WIN32__
+extern "C" void omniORB_rethrow_exception(unsigned, EXCEPTION_POINTERS*)
+{
+  throw;
+}
+static void abortOnNativeExceptionInterceptor(omniInterceptors::
+					      createThread_T::info_T& info)
+{
+  _set_se_translator(omniORB_rethrow_exception);
+  info.run();
+}
+#endif
+
 /////////////////////////////////////////////////////////////////////////////
 //            Module initialiser                                           //
 /////////////////////////////////////////////////////////////////////////////
@@ -1387,10 +1557,13 @@ public:
 
   omni_omniInternal_initialiser() {
     orbOptions::singleton().registerHandler(traceLevelHandler_);
+    orbOptions::singleton().registerHandler(traceExceptionsHandler_);
     orbOptions::singleton().registerHandler(traceInvocationsHandler_);
     orbOptions::singleton().registerHandler(traceThreadIdHandler_);
+    orbOptions::singleton().registerHandler(traceFileHandler_);
     orbOptions::singleton().registerHandler(objectTableSizeHandler_);
     orbOptions::singleton().registerHandler(abortOnInternalErrorHandler_);
+    orbOptions::singleton().registerHandler(abortOnNativeExceptionHandler_);
   }
 
   void attach() {
@@ -1413,6 +1586,13 @@ public:
 
     objectTable = new omniObjTableEntry* [objectTableSize];
     for( CORBA::ULong i = 0; i < objectTableSize; i++ )  objectTable[i] = 0;
+
+#ifdef __WIN32__
+    if (abortOnNativeException) {
+      omniInterceptors* interceptors = omniORB::getInterceptors();
+      interceptors->createThread.add(abortOnNativeExceptionInterceptor);
+    }
+#endif
   }
 
   void detach() {
@@ -1447,13 +1627,13 @@ OMNI_NAMESPACE_END(omni)
 //            Nil object reference list                                    //
 /////////////////////////////////////////////////////////////////////////////
 
-static omnivector<CORBA::Object_ptr>* nilObjectList() {
+static omnivector<CORBA::Object_ptr>*& nilObjectList() {
   static omnivector<CORBA::Object_ptr>* the_list = 0;
   if (!the_list) the_list = new omnivector<CORBA::Object_ptr>;
   return the_list;
 }
 
-static omnivector<omniTrackedObject*>* trackedList() {
+static omnivector<omniTrackedObject*>*& trackedList() {
   static omnivector<omniTrackedObject*>* the_list = 0;
   if (!the_list) the_list = new omnivector<omniTrackedObject*>;
   return the_list;
@@ -1503,6 +1683,7 @@ _omniFinalCleanup::~_omniFinalCleanup()
     delete *i;
 
   delete nilObjectList();
+  nilObjectList() = 0;
 
   int tracked = 0;
   omnivector<omniTrackedObject*>::iterator j = trackedList()->begin();
@@ -1510,6 +1691,7 @@ _omniFinalCleanup::~_omniFinalCleanup()
     delete *j;
 
   delete trackedList();
+  trackedList() = 0;
 
   if (omniORB::trace(15)) {
     omniORB::logger l;
